import { Suspense, lazy, ComponentType, useEffect, startTransition } from "react";
import { Switch, Route, useLocation, Link } from "wouter";
import { QueryClientProvider } from "@tanstack/react-query";
import { Toaster } from "@/components/ui/toaster";
import { queryClient } from "./lib/queryClient";
import { AuthProvider } from "@/hooks/use-auth";
import { useScrollTop } from "@/hooks/use-scroll-top";
import { preloadCriticalResources, deferNonCriticalResources } from "@/lib/optimizations";
import { preloadImages } from "@/lib/image";
import Loading from "@/components/ui/loading";
import { optimizeCriticalPath, deferTask } from "./lib/critical-path";
import { reportWebVitals, markPerformancePoint } from "./lib/performance-utils";
import { AuthGuard } from "@/components/auth/auth-guard";

// Import layout components - these are used on every page so they should be preloaded
import Header from "@/components/layout/simple-header";
import Footer from "@/components/layout/simple-footer";

// Basic wrapper around React.lazy with optional preloading
function lazyImport<T extends ComponentType<any>>(
  factory: () => Promise<{ default: T }>,
  shouldPreload = false
): React.LazyExoticComponent<T> {
  if (shouldPreload) {
    // Start the import immediately
    const promise = factory();
    promise.catch(() => {/* Silently ignore preload errors */});
  }
  
  return lazy(factory);
}

// Only preload truly critical paths
const Home = lazyImport(() => import("@/pages/home"), true);
const Contact = lazyImport(() => import("@/pages/contact"));
const Consultation = lazyImport(() => import("@/pages/consultation"));
const Services = lazyImport(() => import("@/pages/services"));
const About = lazyImport(() => import("@/pages/about"));
const Team = lazyImport(() => import("@/pages/about/team"));
const Sitemap = lazyImport(() => import("@/pages/sitemap"));

// Group routes by category for better code splitting
const serviceRoutes = {
  WillsProbate: lazyImport(() => import("@/pages/services/wills-probate")),
  ExecutorOfWills: lazyImport(() => import("@/pages/services/executor-of-wills")),
  Locations: {
    Stockport: lazyImport(() => import("@/pages/services/locations/stockport-wills-probate")),
    Cheadle: lazyImport(() => import("@/pages/services/locations/cheadle-wills-probate")),
    Bramhall: lazyImport(() => import("@/pages/services/locations/bramhall-wills-probate")),
    Marple: lazyImport(() => import("@/pages/services/locations/marple-wills-probate")),
  },
  Other: {
    FamilyLaw: lazyImport(() => import("@/pages/services/family-law")),
    BusinessLaw: lazyImport(() => import("@/pages/services/business-law")),
    EmploymentLaw: lazyImport(() => import("@/pages/services/employment-law")),
    PersonalInjury: lazyImport(() => import("@/pages/services/personal-injury")),
    Conveyancing: lazyImport(() => import("@/pages/services/conveyancing")),
    LastingPowers: lazyImport(() => import("@/pages/services/lasting-powers"))
  }
};

// Secondary routes
const otherRoutes = {
  NotFound: lazyImport(() => import("@/pages/not-found")),
  TeamMember: lazyImport(() => import("@/pages/team/[name]")),
  Auth: lazyImport(() => import("@/pages/auth")),
  Insights: lazyImport(() => import("@/pages/insights")),
  BlogPost: lazyImport(() => import("@/pages/insights/[slug]")),
  // Add legal routes
  PrivacyPolicy: lazyImport(() => import("@/pages/legal/privacy-policy")),
  Terms: lazyImport(() => import("@/pages/legal/terms"))
};

// Admin routes - load on demand
const adminRoutes = {
  Dashboard: lazyImport(() => import("@/pages/admin/dashboard")),
  Blog: lazyImport(() => import("@/pages/admin/blog")),
  Team: lazyImport(() => import("@/pages/admin/team")),
  Contacts: lazyImport(() => import("@/pages/admin/contacts")),
  Messages: lazyImport(() => import("@/pages/admin/messages")),
  MessageTypeRedirector: lazyImport(() => import("@/pages/admin/messages/[type]")),
  ContactMessage: lazyImport(() => import("@/pages/admin/messages/contact/[id]")),
  ConsultationMessage: lazyImport(() => import("@/pages/admin/messages/consultation/[id]"))
};

// Optimized route configuration
const routes = [
  { path: "/", component: Home },
  { path: "/contact", component: Contact },
  { path: "/consultation", component: Consultation },
  { path: "/consultation-page", component: () => {
    const [, setLocation] = useLocation();
    useEffect(() => {
      setLocation("/consultation");
    }, [setLocation]);
    return null;
  }},
  { path: "/services", component: Services },
  { path: "/about", component: About },
  { path: "/about/team", component: Team },
  { path: "/sitemap", component: Sitemap },
  { path: "/team/:name", component: otherRoutes.TeamMember },
  { path: "/insights", component: otherRoutes.Insights },
  { path: "/insights/:slug", component: otherRoutes.BlogPost },
  { path: "/auth", component: otherRoutes.Auth },

  // Add legal routes
  { path: "/legal/privacy-policy", component: otherRoutes.PrivacyPolicy },
  { path: "/legal/terms", component: otherRoutes.Terms },

  // Service routes
  { path: "/services/wills-probate", component: serviceRoutes.WillsProbate },
  { path: "/services/executor-of-wills", component: serviceRoutes.ExecutorOfWills },
  { path: "/services/conveyancing", component: serviceRoutes.Other.Conveyancing },
  { path: "/services/lasting-powers", component: serviceRoutes.Other.LastingPowers },
  { path: "/services/family-law", component: serviceRoutes.Other.FamilyLaw },
  { path: "/services/business-law", component: serviceRoutes.Other.BusinessLaw },
  { path: "/services/employment-law", component: serviceRoutes.Other.EmploymentLaw },
  { path: "/services/personal-injury", component: serviceRoutes.Other.PersonalInjury },

  // Location-specific routes
  { path: "/services/locations/stockport-wills-probate", component: serviceRoutes.Locations.Stockport },
  { path: "/services/locations/cheadle-wills-probate", component: serviceRoutes.Locations.Cheadle },
  { path: "/services/locations/bramhall-wills-probate", component: serviceRoutes.Locations.Bramhall },
  { path: "/services/locations/marple-wills-probate", component: serviceRoutes.Locations.Marple },

  // Admin routes - most specific routes first
  { path: "/admin/messages/contact/:id", component: adminRoutes.ContactMessage },
  { path: "/admin/messages/consultation/:id", component: adminRoutes.ConsultationMessage },
  { path: "/admin/messages/:type", component: adminRoutes.MessageTypeRedirector },
  { path: "/admin/messages", component: adminRoutes.Messages },
  { path: "/admin/blog", component: adminRoutes.Blog },
  { path: "/admin/team", component: adminRoutes.Team },
  { path: "/admin/contacts", component: adminRoutes.Contacts },
  { path: "/admin", component: adminRoutes.Dashboard },

  // 404 route must be last
  { path: "*", component: otherRoutes.NotFound }
];

function Router() {
  useScrollTop();
  const [location] = useLocation();
  
  // Determine if we're on an admin page
  const isAdminPage = location.startsWith('/admin');

  // Optimized resource preloading and deferral with reduced TBT
  useEffect(() => {
    // Only perform essential work during initial page load
    // Defer all other work to avoid blocking the main thread
    
    // Immediate optimization with minimal work
    deferNonCriticalResources('iframe:not([loading="eager"]), script:not([src*="critical"])');
    
    // Route-specific optimizations - using setTimeout to break up work
    setTimeout(() => {
      try {
        // For images, use a separate defer call after short delay
        deferNonCriticalResources('img:not([loading="eager"]):not([data-priority="true"])');
        
        if (location === '/') {
          // Home page - stagger API loading to reduce TBT impact
          
          // Load only most essential data immediately with low priority
          const servicesPromise = fetch('/api/services')
            .then(res => res.json())
            .catch(() => null); // Fail gracefully
            
          // Prefetch after very minimal delay
          setTimeout(() => {
            queryClient.prefetchQuery({
              queryKey: ['/api/services'],
              queryFn: () => servicesPromise,
              staleTime: 300000 // 5 minutes
            });
          }, 50);
          
          // Much less critical work with longer delay
          setTimeout(() => {
            queryClient.prefetchQuery({
              queryKey: ['/api/team'],
              queryFn: () => fetch('/api/team').then(res => res.json()),
              staleTime: 300000 // 5 minutes
            });
          }, 1000);
        } 
        else if (location.startsWith('/services')) {
          // Delay services data loading
          setTimeout(() => {
            queryClient.prefetchQuery({
              queryKey: ['/api/services'],
              queryFn: () => fetch('/api/services').then(res => res.json()),
              staleTime: 300000 // 5 minutes
            });
          }, 300);
        }
        else if (location === '/about' || location === '/about/team') {
          // Delay team data loading
          setTimeout(() => {
            queryClient.prefetchQuery({
              queryKey: ['/api/team'],
              queryFn: () => fetch('/api/team').then(res => res.json()),
              staleTime: 300000 // 5 minutes
            });
          }, 300);
        }
        else if (location.includes('/insights')) {
          // Delay blog data loading
          setTimeout(() => {
            queryClient.prefetchQuery({
              queryKey: ['blog_posts'],
              queryFn: () => fetch('/api/blog-posts').then(res => res.json()),
              staleTime: 300000 // 5 minutes
            });
          }, 300);
        }
      } catch (error) {
        console.error('Error optimizing resources:', error);
      }
    }, 50); // Brief delay to let initial render complete
  }, [location]);

  // Content to be rendered
  const content = (
    <div className="min-h-screen flex flex-col">
      {/* Since Header is directly imported now, no need for Suspense */}
      {!isAdminPage && <Header />}
      
      <main className="flex-grow">
        {/* Use a more optimized approach for Suspense */}
        {location === '/' ? (
          /* Home page gets special treatment - only fallback briefly 
             and immediately show content when it's ready */
          <Suspense fallback={
            <div className="min-h-screen w-full flex items-center justify-center">
              <div className="w-full h-screen animate-pulse bg-gray-50" />
            </div>
          }>
            <Switch>
              {routes.map(({ path, component: Component }) => (
                <Route key={path} path={path}>
                  {params => <Component {...params} />}
                </Route>
              ))}
            </Switch>
          </Suspense>
        ) : (
          /* Other pages use standard Loading component */
          <Suspense fallback={<Loading />}>
            <Switch>
              {routes.map(({ path, component: Component }) => (
                <Route key={path} path={path}>
                  {params => <Component {...params} />}
                </Route>
              ))}
            </Switch>
          </Suspense>
        )}
      </main>
      
      {/* Since Footer is directly imported now, no need for Suspense */}
      {!isAdminPage && <Footer />}
    </div>
  );

  // Wrap admin routes with AuthGuard
  return isAdminPage ? (
    <AuthGuard forceCheck={true}>
      {content}
    </AuthGuard>
  ) : content;
}

function App() {
  // Apply global performance optimizations when component mounts
  // Split work into multiple phases to minimize TBT
  useEffect(() => {
    // --- PHASE 1: Critical path only ---
    // Only run absolute minimal work needed for initial render
    if (typeof window !== 'undefined') {
      // Mark the start of the app
      markPerformancePoint('app-init-start');
    }
    
    // --- PHASE 2: Minimal delay for important work ---
    // Use requestIdleCallback or setTimeout with minimal delay
    const runPhase2 = () => {
      if (typeof window !== 'undefined') {
        // Initialize the most basic optimizations
        optimizeCriticalPath();
      }
    };
    
    // Use requestIdleCallback if available, or minimal timeout
    if (window.requestIdleCallback) {
      window.requestIdleCallback(runPhase2, { timeout: 100 });
    } else {
      setTimeout(runPhase2, 10);
    }
    
    // --- PHASE 3: Medium delay for less critical work ---
    setTimeout(() => {
      if (typeof window !== 'undefined') {
        // When window loads, report performance metrics
        window.addEventListener('load', () => {
          setTimeout(() => {
            reportWebVitals();
          }, 0);
        });
      }
    }, 200);
    
    // --- PHASE 4: Long delay for non-critical work ---
    setTimeout(() => {
      if (typeof document) {
        // Optimize script loading - lowest priority
        document.querySelectorAll('script').forEach(script => {
          if (!script.hasAttribute('defer') && !script.hasAttribute('async')) {
            script.defer = true;
          }
        });
      }
      
      // Apply further performance optimizations
      deferTask(() => {
        import('./lib/optimizations').then(({ applyPerformanceOptimizations }) => {
          applyPerformanceOptimizations();
        }).catch(() => {/* Silently fail */});
      });
    }, 1000);
  }, []);
  
  return (
    <QueryClientProvider client={queryClient}>
      <AuthProvider>
        <Router />
        <Toaster />
      </AuthProvider>
    </QueryClientProvider>
  );
}

export default App;